#!/bin/sh
#
# innotek VirtualBox
# VirtualBox Guest Additions installation script for Linux

#
#  Copyright (C) 2006-2007 innotek GmbH
# 
#  innotek GmbH confidential
#  All rights reserved

PATH=$PATH:/bin:/sbin:/usr/sbin

. ./routines.sh

# Find the version of X installed
x_version=$(X -version 2>&1 | sed -n 's/^X Window System Version \([0-9.]\+\)/\1/p')$(X -version 2>&1 | sed -n 's/^XFree86 Version \([0-9.]\+\)/\1/p')

build_test_module() {
    # Build a test module and try to load it
    log "Building a test kernel module..."
    log ""
    if ! make --no-print-directory -C module/test >> $LOG 2>&1; then
        info "Building a test kernel module failed.  See the log file $LOG"
        info "for more details."
        log ""
        log "Building the test kernel module failed."
        return 1
    else
        if [ -f module/test/vboxadd_test.ko ]; then
            test_module="module/test/vboxadd_test.ko"
        else
            test_module="module/test/vboxadd_test.o"
        fi
        if [ -f $test_module ]; then
            log "Inserting the test module $test_module into the kernel."
            insmod $test_module >> $LOG 2>&1
            lsmod_line=`lsmod | grep vboxadd_test`
            kernel_name=`expr "$lsmod_line" : '\([^ 	]*\)'`
            if [ "$kernel_name" = "" ]; then
                info "Unable to insert a test module into your Linux kernel.  This may be because"
                info "the kernel was built with a different version of GCC to the one which is"
                info "currently installed, or because the kernel header files are incorrectly"
                info "installed.  The command 'dmesg' may provide you with more information."
                log "Failed to insert the test module."
                return 1
            else
                rmmod $kernel_name > /dev/null 2>&1
                make -C module/test clean > /dev/null 2>&1
            fi
        else
            info "Building a test kernel module failed.  The log file $LOG may provide more"
            info "information."
            log "Could not find the test kernel module we just built."
            return 1
        fi
    fi
    return 0
}

LOG="/var/log/vboxadd-install.log"
VERSION="1.5.0"
BUILD="Mon Sep  3 17:32:26 CEST 2007"
OSE=""

info "VirtualBox $VERSION Guest Additions installation"

check_root

# Initialise logging before all else
create_log $LOG
log "Installing VirtualBox $VERSION Guest Additions, built $BUILD"
log ""

# Parse arguments
install_vfs=""
install_clipboard=""
install_timesync=""
install_xdrivers=""
if [ "$1" = "" ]; then
    install_vfs="yes"
    install_clipboard="yes"
    install_timesync="yes"
    install_xdrivers="yes"
fi
while [ ! -z "$1" ]; do
    case "$1" in
        kernel-module)
            # Nothing to do - we install this whatever happens.
            ;;
        vfs-module)
            install_vfs="yes"
            ;;
        timesync)
            install_timesync="yes"
            ;;
        x11)
            install_xdrivers="yes"
            ;;
        clipboard)
            install_clipboard="yes"
            ;;
        all)
            install_vfs="yes"
            install_timesync="yes"
            install_xdrivers="yes"
            ;;
        help)
            info "Usage: sh VBoxLinuxAdditions.run <options...>"
            info
            info "The following options are recognized:"
            info
            info "  all           -- install all components of the Guest Additions"
            info "  kernel-module -- only install the core components in the Linux kernel"
            info "  vfs-module    -- install the shared folder component"
            info "  timesync      -- install the time synchronisation component"
            info "  x11           -- install guest drivers for the X Window System"
            info
            info "If you do not specify any options, all components will be installed."
            info "Components which you do not install will be deactivated if they are already"
            info "present in order to prevent possible problems due to mixing different versions"
            info "of the Additions."
            exit 1
            ;;
        *)
            info "The option $1 is not known.  You can use any of the following options:"
            info
            info "  all           -- install all components of the Guest Additions"
            info "  kernel-module -- only install the core components in the Linux kernel"
            info "  vfs-module    -- install the shared folder component"
            info "  timesync      -- install the time synchronisation component"
            info "  x11           -- install guest drivers for the X Window System"
            info
            info "If you do not specify any options, all components will be installed."
            info "Components which you do not install will be deactivated if they are already"
            info "present in order to prevent possible problems due to mixing different versions"
            info "of the Additions."
            exit 1
            ;;
    esac
    shift
done

# Check that all the tools we need are available
log "Testing the setup of the guest system"
log ""

have_gmake="`check_gmake; echo $?`"     # Do we have GNU make?
have_ksource="`check_ksource; echo $?`" # Can we find the kernel source?
have_gcc="`check_gcc; echo $?`"         # Is GCC installed?

if [ ! "$install_xdrivers" = "" ]; then
    perl_ok="`check_perl; echo $?`"   # Is perl installed?
else
    perl_ok="-1"                      # We don't need it in this case.
fi # we are installing the X11 drivers

test_build=1
if [ $have_gmake -eq 0 -a $have_ksource -eq 0 -a $have_gcc -eq 0 ]; then
    test_build="`build_test_module; echo $?`"  # Can we build a working kernel module?
fi # no problems with GCC, Make or the kernel headers missing

if [ $perl_ok -eq 1 -o $test_build -eq 1 ]; then
    info "Problems were found which would prevent the Guest Additions from installing."
    info "Please correct these problems and try again."
    log "Giving up due to the problems mentioned above."
    exit 1
fi

# Make kernel module
# What the **** is this for?  I took it from the old Makefile.
pathdown_sh=/lib/modules/`uname -r`/build/scripts/pathdown.sh
if [ -f $pathdown_sh -a ! -x $pathdown_sh ]; then
    chmod +x $pathdown_sh
fi
log ""
log "Building the VirtualBox Guest Additions kernel module."
log ""
echo "Building the VirtualBox Guest Additions kernel module..." 1>&2
if ! module/vboxadd/build_in_tmp \
    --save-module-symvers /var/log/vboxadd-Module.symvers \
    --no-print-directory install >> $LOG 2>&1
then
    info "Unable to build the kernel module.  See the log file $LOG"
    info "for more details."
    exit 1
fi
if [ ! "$install_vfs" = "" ]; then
    log ""
    log "Building the shared folder support kernel module."
    log ""
    echo "Building the shared folder support kernel module..." 1>&2
    if ! module/vboxvfs/build_in_tmp \
        --use-module-symvers /var/log/vboxadd-Module.symvers \
        --no-print-directory install >> $LOG 2>&1
    then
        info "Unable to build the kernel module.  See the log file $LOG"
        info "for more details."
        exit 1
    fi
fi

# Remove previous installations
mod_path=/lib/modules/`uname -r`
find $mod_path -name vboxadd\* | grep -v $mod_path/misc/vboxadd | xargs rm 2> /dev/null
find $mod_path -name vboxvfs\* | grep -v $mod_path/misc/vboxvfs | xargs rm 2> /dev/null
# Get rid of empty directories to which we have installed in the past
rmdir -p $mod_path/misc 2> /dev/null
rmdir -p $mod_path/kernel/misc 2> /dev/null
rmdir -p $mod_path/kernel/drivers/vbox 2> /dev/null
rmdir -p $mod_path/drivers/vbox 2> /dev/null

# Install scripts and binary drivers
info "Installing the VirtualBox Guest Additions..."
info ""
install_init_script vboxadd.sh vboxadd
log "Installing the Guest Additions kernel module system service."
if ! addrunlevel vboxadd 1 99; then
    info "Unable to install Guest Additions kernel module system service."
    log "Failed."
    exit 1
fi
if [ "$install_vfs" = "" ]; then
    # make sure that no old version of the shared folder support is running
    delrunlevel vboxvfs 2 98 > /dev/null 2>&1
else
    log "Installing the shared folder system service."
    install_init_script vboxvfs.sh vboxvfs
    if ! addrunlevel vboxvfs 2 98; then
        info "Unable to install the shared folder system service."
        log "Failed."
        exit 1
    fi
    install -o 0 -g 0 -m 0755 mount.vboxsf /sbin/mount.vboxsf
fi
if [ "$install_timesync" = "" ]; then
    # make sure that no old version of the time synchronisation support is running
    delrunlevel vboxadd-timesync 2 98 > /dev/null 2>&1
else
    log "Installing the time synchronisation system service."
    install -o 0 -g 0 -m 0755 vboxadd-timesync /usr/sbin/vboxadd-timesync
    install_init_script vboxadd-timesync.sh vboxadd-timesync
    if ! addrunlevel vboxadd-timesync 2 98; then
        info "Unable to install the time synchronisation system service."
        log "Failed."
        exit 1
    fi
fi
if [ "$install_clipboard" != "" ]; then
    log "Installing the shared clipboard user service."
    install -o 0 -g 0 -m 04755 vboxadd-xclient /usr/bin/vboxadd-xclient
    if [ ! -z "$x_version" ]; then
        install_x11_startup_app 98vboxadd-xclient "shared clipboard"
    else
        if [ "$install_xdrivers" = "" ]; then
            # If the X drivers are to be installed then the user will be told that X is not there
            # anyway
            info "Could not find XFree86 or Xorg on your guest system.  The shared clipboard"
            info "will not be set up"
        fi
        log "X11 not found - not setting up the service"
    fi
fi

# Create udev description file
if [ -d /etc/udev/rules.d ]; then
    log "Creating udev rule for the Guest Additions kernel module."
    udev_out=`udevinfo -V 2> /dev/null`
    if [ "$udev_out" = "" ]; then
        udev_ver=1
    else
        udev_ver=`expr "$udev_out" : '[^0-9]*\([0-9]*\)'`
    fi
    if [ "$udev_ver" -lt 55 ]; then
        echo "KERNEL=\"vboxadd\", NAME=\"vboxadd\", OWNER=\"root\", MODE=\"0660\"" > /etc/udev/rules.d/60-vboxadd.rules
    else
        echo "KERNEL==\"vboxadd\", NAME=\"vboxadd\", OWNER=\"root\", MODE=\"0660\"" > /etc/udev/rules.d/60-vboxadd.rules
    fi
fi

depmod

# Install the X Window drivers
if [ ! "$install_xdrivers" = "" ]; then
    log "Installing the X Window System drivers"

    for dir in /usr/lib/X11/modules /usr/X11R6/lib/modules; do
	if [ -d $dir ]; then
	    modules_dir=$dir
	    break
	fi
    done

    if [ -z "$x_version" -o -z "$modules_dir" ]; then
            info "Could not find X.org or XFree86 on the guest system.  The X Window drivers"
            info "will not be installed."
            log "Could not find either x.org (/usr/lib/xorg) or XFree86 (/usr/X11R6/lib) on"
            log "  the guest system."
            exit 1
    fi

    case $x_version in
	6.9.* | 7.0.* )
            install -o 0 -g 0 -m 0644 vboxvideo_drv_70.so $modules_dir/drivers/vboxvideo_drv.so
            install -o 0 -g 0 -m 0644 vboxmouse_drv_70.so $modules_dir/input/vboxmouse_drv.so
	    ;;
	7.1.* | 7.2.* )
            install -o 0 -g 0 -m 0644 vboxvideo_drv_71.so $modules_dir/drivers/vboxvideo_drv.so
            install -o 0 -g 0 -m 0644 vboxmouse_drv_71.so $modules_dir/input/vboxmouse_drv.so
	    ;;
	1.3.* )
	    # Xorg have started using the server version number instead of
	    # the release version.
            install -o 0 -g 0 -m 0644 vboxvideo_drv_13.so $modules_dir/drivers/vboxvideo_drv.so
            install -o 0 -g 0 -m 0644 vboxmouse_drv_71.so $modules_dir/input/vboxmouse_drv.so
	    ;;
	* )
	    # Assume X.Org 6.8.x or XFree86
            install -o 0 -g 0 -m 0644 vboxvideo_drv.o $modules_dir/drivers/vboxvideo_drv.o
            install -o 0 -g 0 -m 0644 vboxmouse_drv.o $modules_dir/input/vboxmouse_drv.o
	    ;;
    esac
    ./x11config.pl >> $LOG 2>&1
fi # Install the X11 drivers

info "Successfully installed the VirtualBox Guest Additions."
info "You must restart your guest system in order to complete the installation."
log ""
log "Successfully completed."
log ""
